class Spider:
    CUSTOMIZED = 'customized'
    CONFIGURABLE = 'configurable'


class DedupMethod:
    OVERWRITE = 'overwrite'
    IGNORE = 'ignore'


class DataSourceType:
    MONGO = 'mongo'
    MYSQL = 'mysql'
    POSTGRES = 'postgres'
    SQLSERVER = 'sqlserver'
    KAFKA = 'kafka'
    ELASTICSEARCH = 'es'
